@@ -2,21 +2,21 @@
 rand.walk = function(X, A, Z, theta, tol, minit, maxit, s2, eta.max, V, verbose)
 {   
     eta.range = c(0, eta.max)
-    moller$randWalk(X, A, Z, theta, tol, minit, maxit, s2, eta.range, V, verbose)
+    randWalk(X, A, Z, theta, tol, minit, maxit, s2, eta.range, V, verbose)
 }
 
-rand.walk.train = function(X, A, Z, theta, trainit, s2, eta.max, V)
+rand.walk.train = function(X, A, Z, theta, trainit, s2, eta.max, V, verbose)
 {   
     eta.range = c(0, eta.max)
-    moller$randWalkTrain(X, A, Z, theta, trainit, s2, eta.range, V)
+    randWalkTrain(X, A, Z, theta, trainit, s2, eta.range, V, verbose)
 }
 
-Moller.train = function(X, A, Z, theta, trainit, s2, eta.max)
+Moller.train = function(X, A, Z, theta, trainit, s2, eta.max, verbose)
 {
     p = length(theta)
     V.init = diag(0.01, p)
     V.init[-p, -p] = vcov(glm(Z ~ X - 1, family = binomial))
-    sample = rand.walk.train(X, A, Z, theta, trainit, s2, eta.max, V.init)
+    sample = rand.walk.train(X, A, Z, theta, trainit, s2, eta.max, V.init, verbose)
     V = cov(sample)
     V
 }
@@ -27,7 +27,7 @@ Moller.run = function(X, A, Z, theta, trainit, tol, minit, maxit, s2, eta.max, v
     mcse = numeric(p)
     coefficients = numeric(p)
     result = list()
-    V = Moller.train(X, A, Z, theta, trainit, s2, eta.max)
+    V = Moller.train(X, A, Z, theta, trainit, s2, eta.max, verbose)
     sample = data.frame(rand.walk(X, A, Z, theta, tol, minit, maxit, s2, eta.max, V, verbose))
     for (i in 1:p)
     {
